@@ -19,11 +19,11 @@ static char const data[][150]={
 namespace {
 struct ObjectCache
 {
-	CRefcountObject<wxString> const& get(wxString const& v)
+	CRefcountObject<fzstring> const& get(fzstring const& v)
 	{
-		auto it = std::lower_bound( cache.begin(), cache.end(), v );
+		auto it = std::lower_bound(cache.begin(), cache.end(), v);
 
-		if( it == cache.end() || !(*it == v) ) {
+		if (it == cache.end() || !(*it == v)) {
 			it = cache.emplace(it, v);
 		}
 		return *it;
@@ -32,14 +32,14 @@ struct ObjectCache
 	// Vector coupled with binary search and sorted insertion is fastest
 	// alternative as we expect a relatively low amount of inserts.
 	// Note that we cannot use set, as it it cannot search based on a different type.
-	std::vector<CRefcountObject<wxString>> cache;
+	std::vector<CRefcountObject<fzstring>> cache;
 };
 
 
 ObjectCache objcache;
 }
 
-class CToken
+class CToken final
 {
 protected:
 	enum TokenInformation
@@ -50,15 +50,7 @@ protected:
 	};
 
 public:
-	CToken()
-		: m_pToken()
-		, m_len()
-		, m_numeric(Unknown)
-		, m_leftNumeric(Unknown)
-		, m_rightNumeric(Unknown)
-		, m_number(-1)
-	{
-	}
+	CToken() = default;
 
 	enum t_numberBase
 	{
@@ -66,16 +58,12 @@ public:
 		hex
 	};
 
-	CToken(const wxChar* p, unsigned int len)
+	CToken(wxChar const* p, unsigned int len)
 		: m_pToken(p)
 		, m_len(len)
-		, m_numeric(Unknown)
-		, m_leftNumeric(Unknown)
-		, m_rightNumeric(Unknown)
-		, m_number(-1)
 	{}
 
-	const wxChar* GetToken() const
+	wxChar const* GetToken() const
 	{
 		return m_pToken;
 	}
@@ -85,16 +73,14 @@ public:
 		return m_len;
 	}
 
-	wxString GetString()
+	fzstring GetString() const
 	{
-		if (!m_pToken)
-			return wxString();
-
-		if( m_str.empty() ) {
-			m_str.assign(m_pToken, m_len);
+		if (!m_pToken || !m_len) {
+			return fzstring();
+		}
+		else {
+			return fzstring(m_pToken, m_len);
 		}
-
-		return m_str;
 	}
 
 	bool IsNumeric(t_numberBase base = decimal)
@@ -135,8 +121,7 @@ public:
 
 	bool IsLeftNumeric()
 	{
-		if (m_leftNumeric == Unknown)
-		{
+		if (m_leftNumeric == Unknown) {
 			if (m_len < 2)
 				m_leftNumeric = No;
 			else if (m_pToken[0] < '0' || m_pToken[0] > '9')
@@ -149,8 +134,7 @@ public:
 
 	bool IsRightNumeric()
 	{
-		if (m_rightNumeric == Unknown)
-		{
+		if (m_rightNumeric == Unknown) {
 			if (m_len < 2)
 				m_rightNumeric = No;
 			else if (m_pToken[m_len - 1] < '0' || m_pToken[m_len - 1] > '9')
@@ -166,10 +150,8 @@ public:
 		if (!chr)
 			return -1;
 
-		for (unsigned int i = start; i < m_len; ++i)
-		{
-			for (int c = 0; chr[c]; ++c)
-			{
+		for (unsigned int i = start; i < m_len; ++i) {
+			for (int c = 0; chr[c]; ++c) {
 				if (m_pToken[i] == chr[c])
 					return i;
 			}
@@ -189,7 +171,7 @@ public:
 		return -1;
 	}
 
-	wxLongLong GetNumber(unsigned int start, int len)
+	int64_t GetNumber(unsigned int start, int len)
 	{
 		if (len == -1)
 			len = m_len - start;
@@ -202,7 +184,7 @@ public:
 		if (m_pToken[start] < '0' || m_pToken[start] > '9')
 			return -1;
 
-		wxLongLong number = 0;
+		int64_t number = 0;
 		for (unsigned int i = start; i < (start + len); ++i)
 		{
 			if (m_pToken[i] < '0' || m_pToken[i] > '9')
@@ -213,27 +195,22 @@ public:
 		return number;
 	}
 
-	wxLongLong GetNumber(t_numberBase base = decimal)
+	int64_t GetNumber(t_numberBase base = decimal)
 	{
-		switch (base)
-		{
+		switch (base) {
 		default:
 		case decimal:
-			if (m_number == -1)
-			{
-				if (IsNumeric() || IsLeftNumeric())
-				{
+			if (m_number == -1) {
+				if (IsNumeric() || IsLeftNumeric()) {
 					m_number = 0;
-					for (unsigned int i = 0; i < m_len; ++i)
-					{
+					for (unsigned int i = 0; i < m_len; ++i) {
 						if (m_pToken[i] < '0' || m_pToken[i] > '9')
 							break;
 						m_number *= 10;
 						m_number += m_pToken[i] - '0';
 					}
 				}
-				else if (IsRightNumeric())
-				{
+				else if (IsRightNumeric()) {
 					m_number = 0;
 					int start = m_len - 1;
 					while (m_pToken[start - 1] >= '0' && m_pToken[start - 1] <= '9')
@@ -248,22 +225,18 @@ public:
 			return m_number;
 		case hex:
 			{
-				wxLongLong number = 0;
-				for (unsigned int i = 0; i < m_len; ++i)
-				{
+				int64_t number = 0;
+				for (unsigned int i = 0; i < m_len; ++i) {
 					const wxChar& c = m_pToken[i];
-					if (c >= '0' && c <= '9')
-					{
+					if (c >= '0' && c <= '9') {
 						number *= 16;
 						number += c - '0';
 					}
-					else if (c >= 'a' && c <= 'f')
-					{
+					else if (c >= 'a' && c <= 'f') {
 						number *= 16;
 						number += c - '0' + 10;
 					}
-					else if (c >= 'A' && c <= 'F')
-					{
+					else if (c >= 'A' && c <= 'F') {
 						number *= 16;
 						number += c - 'A' + 10;
 					}
@@ -284,14 +257,13 @@ public:
 	}
 
 protected:
-	const wxChar* m_pToken;
-	unsigned int m_len;
+	wxChar const* m_pToken{};
+	unsigned int m_len{};
 
-	TokenInformation m_numeric;
-	TokenInformation m_leftNumeric;
-	TokenInformation m_rightNumeric;
-	wxLongLong m_number;
-	wxString m_str;
+	TokenInformation m_numeric{Unknown};
+	TokenInformation m_leftNumeric{Unknown};
+	TokenInformation m_rightNumeric{Unknown};
+	int64_t m_number{-1};
 };
 
 class CLine
@@ -300,7 +272,7 @@ public:
 	CLine(wxChar* p, int len = -1, int trailing_whitespace = 0)
 	{
 		m_pLine = p;
-		if (len != -1)
+		if (len >= 0)
 			m_len = len;
 		else
 			m_len = wxStrlen(p);
@@ -364,8 +336,6 @@ public:
 		}
 		else {
 			if (include_whitespace) {
-				const wxChar* p;
-
 				int prev = n - offset_;
 				if (prev)
 					--prev;
@@ -373,9 +343,13 @@ public:
 				CToken ref;
 				if (!GetToken(prev, ref))
 					return false;
-				p = ref.GetToken() + ref.GetLength() + 1;
+				wxChar const* p = ref.GetToken() + ref.GetLength() + 1;
 
-				token = CToken(p, m_len - (p - m_pLine));
+				auto const newLen = m_len - (p - m_pLine);
+				if (newLen <= 0) {
+					return false;
+				}
+				token = CToken(p, newLen);
 				return true;
 			}
 
@@ -391,7 +365,11 @@ public:
 			for (unsigned int i = static_cast<unsigned int>(m_LineEndTokens.size()); i <= n; ++i) {
 				const CToken *refToken = m_Tokens[i];
 				const wxChar* p = refToken->GetToken();
-				CToken *pToken = new CToken(p, m_len - (p - m_pLine) - m_trailing_whitespace);
+				auto const newLen = m_len - (p - m_pLine) - m_trailing_whitespace;
+				if (newLen <= 0) {
+					return false;
+				}
+				CToken *pToken = new CToken(p, newLen);
 				m_LineEndTokens.push_back(pToken);
 			}
 			token = *(m_LineEndTokens[n]);
@@ -854,9 +832,9 @@ done:
 
 	if (serverType == VMS && entry.is_dir()) {
 		// Trim version information from directories
-		int pos = entry.name.Find(';', true);
-		if (pos > 0)
-			entry.name = entry.name.Left(pos);
+		auto pos = entry.name.rfind(';');
+		if (pos != fzstring::npos && pos > 0)
+			entry.name = entry.name.substr(0, pos);
 	}
 
 	if (sftp_mode_) {
@@ -864,9 +842,9 @@ done:
 
 		CToken t;
 		if (line.GetToken(0, t)) {
-			wxLongLong seconds = t.GetNumber();
-			if (seconds > 0 && seconds.GetValue() <= 0xffffffffll) {
-				CDateTime time(static_cast<time_t>(seconds.GetValue()), CDateTime::seconds);
+			int64_t seconds = t.GetNumber();
+			if (seconds > 0 && seconds <= 0xffffffffll) {
+				CDateTime time(static_cast<time_t>(seconds), CDateTime::seconds);
 				if (time.IsValid()) {
 					entry.time = time;
 				}
@@ -908,7 +886,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		chr != '-')
 		return false;
 
-	wxString permissions = token.GetString();
+	fzstring permissions = token.GetString();
 
 	entry.flags = 0;
 
@@ -926,7 +904,6 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		permissions += _T(" ") + token.GetString();
 		netware = true;
 	}
-	entry.permissions = objcache.get(permissions);
 
 	int numOwnerGroup = 3;
 	if (!netware) {
@@ -944,7 +921,7 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 		// Reset index
 		index = startindex;
 
-		wxString ownerGroup;
+		fzstring ownerGroup;
 		for (int i = 0; i < numOwnerGroup; ++i) {
 			if (!line.GetToken(++index, token))
 				return false;
@@ -989,23 +966,24 @@ bool CDirectoryListingParser::ParseAsUnix(CLine &line, CDirentry &entry, bool ex
 
 		entry.name = token.GetString();
 
-		// Filter out cpecial chars at the end of the filenames
+		// Filter out special chars at the end of the filenames
 		chr = token[token.GetLength() - 1];
 		if (chr == '/' ||
 			chr == '|' ||
 			chr == '*')
-			entry.name.RemoveLast();
+			entry.name.pop_back();
 
 		if (entry.is_link()) {
-			int pos;
-			if ((pos = entry.name.Find(_T(" -> "))) != -1) {
-				entry.target = CSparseOptional<wxString>(entry.name.Mid(pos + 4));
-				entry.name = entry.name.Left(pos);
+			size_t pos;
+			if ((pos = entry.name.find(_T(" -> "))) != fzstring::npos) {
+				entry.target = CSparseOptional<fzstring>(entry.name.substr(pos + 4));
+				entry.name = entry.name.substr(0, pos);
 			}
 		}
 
 		entry.time += m_timezoneOffset;
 
+		entry.permissions = objcache.get(permissions);
 		entry.ownerGroup = objcache.get(ownerGroup);
 		return true;
 	}
@@ -1041,7 +1019,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		if (pos2 == -1) {
 			if (token[pos] != '.') {
 				// something like 26-05 2002
-				day = token.GetNumber(pos + 1, token.GetLength() - pos - 1).GetLo();
+				day = token.GetNumber(pos + 1, token.GetLength() - pos - 1);
 				if (day < 1 || day > 31)
 					return false;
 				dateMonth = CToken(token.GetToken(), pos);
@@ -1070,7 +1048,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 			// 1) 2005 3 13
 			// 2) 2005 13 3
 			// assume first one.
-			year = token.GetNumber().GetLo();
+			year = token.GetNumber();
 			if (!line.GetToken(++index, dateMonth))
 				return false;
 			mayHaveTime = false;
@@ -1086,7 +1064,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 				return false;
 
 			// Asian date format: 2005xxx 5xx 20xxx with some non-ascii characters following
-			year = token.GetNumber().GetLo();
+			year = token.GetNumber();
 			if (!line.GetToken(++index, dateMonth))
 				return false;
 			mayHaveTime = false;
@@ -1105,11 +1083,11 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		// Check for non-numeric day
 		if (!token.IsNumeric() && !token.IsLeftNumeric()) {
 			int offset = 0;
-			if (dateMonth.GetString().Right(1) == _T("."))
+			if (dateMonth.GetString().back() == '.')
 				++offset;
 			if (!dateMonth.IsNumeric(0, dateMonth.GetLength() - offset))
 				return false;
-			dateDay = dateMonth.GetNumber(0, dateMonth.GetLength() - offset).GetLo();
+			dateDay = dateMonth.GetNumber(0, dateMonth.GetLength() - offset);
 			dateMonth = token;
 		}
 		else if( token.GetLength() == 5 && token[2] == ':' && token.IsRightNumeric() ) {
@@ -1117,7 +1095,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 			return false;
 		}
 		else {
-			dateDay = token.GetNumber().GetLo();
+			dateDay = token.GetNumber();
 			if (token[token.GetLength() - 1] == ',')
 				bHasYearAndTime = true;
 		}
@@ -1190,7 +1168,7 @@ bool CDirectoryListingParser::ParseUnixDateTime(CLine & line, int &index, CDiren
 		if (!token.IsNumeric() && !token.IsLeftNumeric())
 			return false;
 
-		year = token.GetNumber().GetLo();
+		year = token.GetNumber();
 
 		if (year > 3000)
 			return false;
@@ -1255,7 +1233,7 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 		// Seems to be monthname-dd-yy
 
 		// Check month name
-		wxString dateMonth = token.GetString().Mid(0, pos);
+		wxString dateMonth = token.GetString().substr(0, pos);
 		if (!GetMonthFromName(dateMonth, month))
 			return false;
 		gotMonth = true;
@@ -1263,23 +1241,23 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	}
 	else if (pos == 4) {
 		// Seems to be yyyy-mm-dd
-		year = token.GetNumber(0, pos).GetLo();
+		year = token.GetNumber(0, pos);
 		if (year < 1900 || year > 3000)
 			return false;
 		gotYear = true;
 	}
 	else if (pos <= 2) {
-		wxLongLong value = token.GetNumber(0, pos);
+		int64_t value = token.GetNumber(0, pos);
 		if (token[pos] == '.') {
 			// Maybe dd.mm.yyyy
 			if (value < 1 || value > 31)
 				return false;
-			day = value.GetLo();
+			day = value;
 			gotDay = true;
 		}
 		else {
 			if (saneFieldOrder) {
-				year = value.GetLo();
+				year = value;
 				if (year < 50)
 					year += 2000;
 				else
@@ -1295,11 +1273,11 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 					if (value > 31)
 						return false;
 
-					day = value.GetLo();
+					day = value;
 					gotDay = true;
 				}
 				else {
-					month = value.GetLo();
+					month = value;
 					gotMonth = true;
 				}
 			}
@@ -1331,21 +1309,21 @@ bool CDirectoryListingParser::ParseShortDate(CToken &token, CDirentry &entry, bo
 	if (gotYear || gotDay) {
 		// Month field in yyyy-mm-dd or dd-mm-yyyy
 		// Check month name
-		wxString dateMonth = token.GetString().Mid(pos + 1, pos2 - pos - 1);
+		wxString dateMonth = token.GetString().substr(pos + 1, pos2 - pos - 1);
 		if (!GetMonthFromName(dateMonth, month))
 			return false;
 		gotMonth = true;
 	}
 	else {
-		wxLongLong value = token.GetNumber(pos + 1, pos2 - pos - 1);
+		int64_t value = token.GetNumber(pos + 1, pos2 - pos - 1);
 		// Day field in mm-dd-yyyy
 		if (value < 1 || value > 31)
 			return false;
-		day = value.GetLo();
+		day = value;
 		gotDay = true;
 	}
 
-	int64_t value = token.GetNumber(pos2 + 1, token.GetLength() - pos2 - 1).GetValue();
+	int64_t value = token.GetNumber(pos2 + 1, token.GetLength() - pos2 - 1);
 	if (gotYear) {
 		// Day field in yyy-mm-dd
 		if (value <= 0 || value > 31)
@@ -1403,18 +1381,15 @@ bool CDirectoryListingParser::ParseAsDos(CLine &line, CDirentry &entry)
 	if (!line.GetToken(++index, token))
 		return false;
 
-	if (token.GetString() == _T("<DIR>"))
-	{
+	if (token.GetString() == _T("<DIR>")) {
 		entry.flags |= CDirentry::flag_dir;
 		entry.size = -1;
 	}
-	else if (token.IsNumeric() || token.IsLeftNumeric())
-	{
+	else if (token.IsNumeric() || token.IsLeftNumeric()) {
 		// Convert size, filter out separators
-		wxLongLong size = 0;
+		int64_t size = 0;
 		int len = token.GetLength();
-		for (int i = 0; i < len; ++i)
-		{
+		for (int i = 0; i < len; ++i) {
 			char chr = token[i];
 			if (chr == ',' || chr == '.')
 				continue;
@@ -1435,8 +1410,8 @@ bool CDirectoryListingParser::ParseAsDos(CLine &line, CDirentry &entry)
 	entry.name = token.GetString();
 
 	entry.target.clear();
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
 	return true;
@@ -1451,7 +1426,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 	if (pos < 1 || static_cast<unsigned int>(pos) >= (token.GetLength() - 1))
 		return false;
 
-	wxLongLong hour = token.GetNumber(0, pos);
+	int64_t hour = token.GetNumber(0, pos);
 	if (hour < 0 || hour > 23)
 		return false;
 
@@ -1466,11 +1441,11 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 	if (!len)
 		return false;
 
-	wxLongLong minute = token.GetNumber(pos + 1, len);
+	int64_t minute = token.GetNumber(pos + 1, len);
 	if (minute < 0 || minute > 59)
 		return false;
 
-	wxLongLong seconds = -1;
+	int64_t seconds = -1;
 	if (pos2 != -1) {
 		// Parse seconds
 		seconds = token.GetNumber(pos2 + 1, -1);
@@ -1479,10 +1454,8 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 	}
 
 	// Convert to 24h format
-	if (!token.IsRightNumeric())
-	{
-		if (token[token.GetLength() - 2] == 'P')
-		{
+	if (!token.IsRightNumeric()) {
+		if (token[token.GetLength() - 2] == 'P') {
 			if (hour < 12)
 				hour += 12;
 		}
@@ -1491,7 +1464,7 @@ bool CDirectoryListingParser::ParseTime(CToken &token, CDirentry &entry)
 				hour = 0;
 	}
 
-	return entry.time.ImbueTime( hour.GetLo(), minute.GetLo(), seconds.GetLo() );
+	return entry.time.ImbueTime(hour, minute, seconds);
 }
 
 bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
@@ -1507,16 +1480,15 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 	if (pos == -1 || static_cast<size_t>(pos) == (token.GetLength() - 1))
 		return false;
 
-	entry.name = token.GetString().Mid(pos + 1);
+	entry.name = token.GetString().substr(pos + 1);
 
 	entry.flags = 0;
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
 	entry.size = -1;
 
+	fzstring permissions;
+
 	int fact = 1;
-	while (fact < pos)
-	{
+	while (fact < pos) {
 		int separator = token.Find(',', fact);
 		int len;
 		if (separator == -1)
@@ -1524,8 +1496,7 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 		else
 			len = separator - fact;
 
-		if (!len)
-		{
+		if (!len) {
 			++fact;
 			continue;
 		}
@@ -1536,39 +1507,42 @@ bool CDirectoryListingParser::ParseAsEplf(CLine &line, CDirentry &entry)
 			entry.flags |= CDirentry::flag_dir;
 		else if (type == 's')
 			entry.size = token.GetNumber(fact + 1, len - 1);
-		else if (type == 'm')
-		{
-			wxLongLong number = token.GetNumber(fact + 1, len - 1);
+		else if (type == 'm') {
+			int64_t number = token.GetNumber(fact + 1, len - 1);
 			if (number < 0)
 				return false;
-			entry.time = CDateTime(static_cast<time_t>(number.GetValue()), CDateTime::seconds);
+			entry.time = CDateTime(static_cast<time_t>(number), CDateTime::seconds);
 		}
 		else if (type == 'u' && len > 2 && token[fact + 1] == 'p')
-			entry.permissions = objcache.get(token.GetString().Mid(fact + 2, len - 2));
+			permissions = token.GetString().substr(fact + 2, len - 2);
 
 		fact += len + 1;
 	}
 
+	entry.permissions = objcache.get(permissions);
+	entry.ownerGroup = objcache.get(fzstring());
 	return true;
 }
 
+namespace {
 wxString Unescape(const wxString& str, wxChar escape)
 {
 	wxString res;
-	for (unsigned int i = 0; i < str.Len(); ++i)
-	{
+	for (unsigned int i = 0; i < str.size(); ++i) {
 		wxChar c = str[i];
-		if (c == escape)
-		{
-			c = str[++i];
-			if (!c)
+		if (c == escape) {
+			++i;
+			if (i == str.size() || !str[i]) {
 				break;
+			}
+			c = str[i];
 		}
 		res += c;
 	}
 
 	return res;
 }
+}
 
 bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 {
@@ -1584,12 +1558,12 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 
 	entry.flags = 0;
 
-	if (pos > 4 && token.GetString().Mid(pos - 4, 4) == _T(".DIR")) {
+	if (pos > 4 && token.GetString().substr(pos - 4, 4) == _T(".DIR")) {
 		entry.flags |= CDirentry::flag_dir;
-		if (token.GetString().Mid(pos) == _T(";1"))
-			entry.name = token.GetString().Left(pos - 4);
+		if (token.GetString().substr(pos) == _T(";1"))
+			entry.name = token.GetString().substr(0, pos - 4);
 		else
-			entry.name = token.GetString().Left(pos - 4) + token.GetString().Mid(pos);
+			entry.name = token.GetString().substr(0, pos - 4) + token.GetString().substr(pos);
 	}
 	else
 		entry.name = token.GetString();
@@ -1600,8 +1574,8 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 	if (!line.GetToken(++index, token))
 		return false;
 
-	wxString ownerGroup;
-	wxString permissions;
+	fzstring ownerGroup;
+	fzstring permissions;
 
 	// This field can either be the filesize, a username (at least that's what I think) enclosed in [] or a date.
 	if (!token.IsNumeric() && !token.IsLeftNumeric()) {
@@ -1609,7 +1583,7 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 		const int len = token.GetLength();
 		if (len < 3 || token[0] != '[' || token[len - 1] != ']')
 			return false;
-		ownerGroup = token.GetString().Mid(1, len - 2);
+		ownerGroup = token.GetString().substr(1, len - 2);
 
 		if (!line.GetToken(++index, token))
 			return false;
@@ -1696,12 +1670,12 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 		if (len > 2 && token[0] == '(' && token[len - 1] == ')') {
 			if (!permissions.empty())
 				permissions += _T(" ");
-			permissions += token.GetString().Mid(1, len - 2);
+			permissions += token.GetString().substr(1, len - 2);
 		}
 		else if (len > 2 && token[0] == '[' && token[len - 1] == ']') {
 			if (!ownerGroup.empty())
 				ownerGroup += _T(" ");
-			ownerGroup += token.GetString().Mid(1, len - 2);
+			ownerGroup += token.GetString().substr(1, len - 2);
 		}
 		else {
 			if (!ownerGroup.empty())
@@ -1720,15 +1694,14 @@ bool CDirectoryListingParser::ParseAsVms(CLine &line, CDirentry &entry)
 bool CDirectoryListingParser::ParseAsIbm(CLine &line, CDirentry &entry)
 {
 	int index = 0;
-	CToken token;
 
 	// Get owner
-	if (!line.GetToken(index, token))
+	CToken ownerGroupToken;
+	if (!line.GetToken(index, ownerGroupToken))
 		return false;
 
-	entry.ownerGroup = objcache.get(token.GetString());
-
 	// Get size
+	CToken token;
 	if (!line.GetToken(++index, token))
 		return false;
 
@@ -1759,10 +1732,13 @@ bool CDirectoryListingParser::ParseAsIbm(CLine &line, CDirentry &entry)
 
 	entry.name = token.GetString();
 	if (token[token.GetLength() - 1] == '/') {
-		entry.name.RemoveLast();
+		entry.name.pop_back();
 		entry.flags |= CDirentry::flag_dir;
 	}
 
+	entry.ownerGroup = objcache.get(ownerGroupToken.GetString());
+	entry.permissions = objcache.get(fzstring());
+
 	entry.time += m_timezoneOffset;
 
 	return true;
@@ -1790,17 +1766,15 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 	// If token is a number, than it's the numerical Unix style format,
 	// else it's the VShell, OS/2 or nortel.VxWorks format
 	if (token.IsNumeric()) {
-		entry.permissions = objcache.get(firstToken.GetString());
 		if (firstToken.GetLength() >= 2 && firstToken[1] == '4')
 			entry.flags |= CDirentry::flag_dir;
 
-		wxString ownerGroup = token.GetString();
+		fzstring ownerGroup = token.GetString();
 
 		if (!line.GetToken(++index, token))
 			return false;
 
 		ownerGroup += _T(" ") + token.GetString();
-		entry.ownerGroup = objcache.get(ownerGroup);
 
 		// Get size
 		if (!line.GetToken(++index, token))
@@ -1815,10 +1789,10 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 		if (!line.GetToken(++index, token))
 			return false;
 
-		wxLongLong number = token.GetNumber();
+		int64_t number = token.GetNumber();
 		if (number < 0)
 			return false;
-		entry.time = CDateTime(static_cast<time_t>(number.GetValue()), CDateTime::seconds);
+		entry.time = CDateTime(static_cast<time_t>(number), CDateTime::seconds);
 
 		// Get filename
 		if (!line.GetToken(++index, token, true))
@@ -1826,6 +1800,9 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 
 		entry.name = token.GetString();
 		entry.target.clear();
+
+		entry.permissions = objcache.get(firstToken.GetString());
+		entry.ownerGroup = objcache.get(ownerGroup);
 	}
 	else {
 		// Possible conflict with multiline VMS listings
@@ -1868,13 +1845,15 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 				return false;
 
 			entry.name = token.GetString();
-			wxString type = entry.name.Right(5);
-			MakeLowerAscii(type);
-			if (!skippedCount && type == _T("<dir>")) {
-				entry.flags |= CDirentry::flag_dir;
-				entry.name = entry.name.Left(entry.name.Length() - 5);
-				while (!entry.name.empty() && entry.name.Last() == ' ')
-					entry.name.RemoveLast();
+			if (entry.name.size() >= 5) {
+				wxString type = entry.name.substr(entry.name.size() - 5);
+				MakeLowerAscii(type);
+				if (!skippedCount && type == _T("<dir>")) {
+					entry.flags |= CDirentry::flag_dir;
+					entry.name = entry.name.substr(0, entry.name.size() - 5);
+					while (!entry.name.empty() && entry.name.back() == ' ')
+						entry.name.pop_back();
+				}
 			}
 		}
 		else {
@@ -1885,7 +1864,7 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 			if (!token.IsNumeric() && !token.IsLeftNumeric())
 				return false;
 
-			wxLongLong day = token.GetNumber();
+			int64_t day = token.GetNumber();
 			if (day < 0 || day > 31)
 				return false;
 
@@ -1896,13 +1875,13 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 			if (!token.IsNumeric())
 				return false;
 
-			wxLongLong year = token.GetNumber();
+			int64_t year = token.GetNumber();
 			if (year < 50)
 				year += 2000;
 			else if (year < 1000)
 				year += 1900;
 
-			if( !entry.time.Set(CDateTime::utc, year.GetLo(), month, day.GetLo()) ) {
+			if( !entry.time.Set(CDateTime::utc, year, month, day) ) {
 				return false;
 			}
 
@@ -1919,15 +1898,14 @@ bool CDirectoryListingParser::ParseOther(CLine &line, CDirentry &entry)
 
 			entry.name = token.GetString();
 			char chr = token[token.GetLength() - 1];
-			if (chr == '/' || chr == '\\')
-			{
+			if (chr == '/' || chr == '\\') {
 				entry.flags |= CDirentry::flag_dir;
-				entry.name.RemoveLast();
+				entry.name.pop_back();
 			}
 		}
 		entry.target.clear();
-		entry.ownerGroup = objcache.get(wxString());
-		entry.permissions = objcache.get(wxString());
+		entry.ownerGroup = objcache.get(fzstring());
+		entry.permissions = entry.ownerGroup;
 		entry.time += m_timezoneOffset;
 	}
 
@@ -1977,7 +1955,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		// Trim empty lines and spaces
 		auto iter = m_DataList.begin();
 		int len = iter->len;
-		while (iter->p[m_currentOffset] == '\r' || iter->p[m_currentOffset] == '\n' || iter->p[m_currentOffset] == ' ' || iter->p[m_currentOffset] == '\t') {
+		while (iter->p[m_currentOffset] == '\r' || iter->p[m_currentOffset] == '\n' || iter->p[m_currentOffset] == ' ' || iter->p[m_currentOffset] == '\t' || !iter->p[m_currentOffset]) {
 			++m_currentOffset;
 			if (m_currentOffset >= len) {
 				delete [] iter->p;
@@ -2000,7 +1978,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 		int emptylen = 0;
 
 		int currentOffset = m_currentOffset;
-		while ((iter->p[currentOffset] != '\n') && (iter->p[currentOffset] != '\r')) {
+		while (iter->p[currentOffset] != '\n' && iter->p[currentOffset] != '\r' && iter->p[currentOffset]) {
 			if (iter->p[currentOffset] == ' ' || iter->p[currentOffset] == '\t')
 				++emptylen;
 			else
@@ -2053,7 +2031,7 @@ CLine *CDirectoryListingParser::GetLine(bool breakAtEnd /*=false*/, bool &error)
 			delete [] i->p;
 			++i;
 		}
-
+		;
 		// Copy last chunk
 		if (iter != m_DataList.end() && reslen) {
 			int copylen = m_currentOffset-startpos;
@@ -2134,7 +2112,7 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine &line, CDirentry &entry)
 	if (!line.GetToken(index++, token))
 		return false;
 
-	if (token.GetString().Right(1) != _T("."))
+	if (token.GetString().back() != '.')
 		return false;
 
 	// Parse time
@@ -2144,8 +2122,8 @@ bool CDirectoryListingParser::ParseAsWfFtp(CLine &line, CDirentry &entry)
 	if (!ParseTime(token, entry))
 		return false;
 
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
 	return true;
@@ -2169,8 +2147,7 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 		return false;
 
 	entry.flags = 0;
-	if (token.GetString() != _T("**NONE**") && !ParseShortDate(token, entry))
-	{
+	if (token.GetString() != _T("**NONE**") && !ParseShortDate(token, entry)) {
 		// Perhaps of the following type:
 		// TSO004 3390 VSAM FOO.BAR
 		if (token.GetString() != _T("VSAM"))
@@ -2180,12 +2157,12 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 			return false;
 
 		entry.name = token.GetString();
-		if (entry.name.Find(' ') != -1)
+		if (entry.name.find(' ') != fzstring::npos)
 			return false;
 
 		entry.size = -1;
-		entry.ownerGroup = objcache.get(wxString());
-		entry.permissions = objcache.get(wxString());
+		entry.ownerGroup = objcache.get(fzstring());
+		entry.permissions = entry.ownerGroup;
 
 		return true;
 	}
@@ -2201,16 +2178,14 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 	// used
 	if (!line.GetToken(index++, token))
 		return false;
-	if (token.IsNumeric() || token.GetString() == _T("????") || token.GetString() == _T("++++") )
-	{
+	if (token.IsNumeric() || token.GetString() == _T("????") || token.GetString() == _T("++++") ) {
 		// recfm
 		if (!line.GetToken(index++, token))
 			return false;
 		if (token.IsNumeric())
 			return false;
 	}
-	else
-	{
+	else {
 		if (prevLen < 6)
 			return false;
 	}
@@ -2245,8 +2220,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS(CLine &line, CDirentry &entry)
 
 	entry.name = token.GetString();
 
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
 
 	return true;
 }
@@ -2308,8 +2283,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS(CLine &line, CDirentry &entry)
 	if (!line.GetToken(index++, token, true))
 		return false;
 
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
 	entry.time += m_timezoneOffset;
 
 	return true;
@@ -2325,7 +2300,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &en
 	if (!line.GetToken(index, token))
 		return false;
 
-	if (token.GetString().CmpNoCase(_T("Migrated")))
+	wxString s = token.GetString();
+	MakeLowerAscii(s);
+	if (s != _T("migrated"))
 		return false;
 
 	if (!line.GetToken(++index, token))
@@ -2333,14 +2310,14 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Migrated(CLine &line, CDirentry &en
 
 	entry.name = token.GetString();
 
-	entry.flags = 0;
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
-	entry.size = -1;
-
 	if (line.GetToken(++index, token))
 		return false;
 
+	entry.flags = 0;
+	entry.size = -1;
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
+
 	return true;
 }
 
@@ -2354,8 +2331,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_PDS2(CLine &line, CDirentry &entry)
 	entry.name = token.GetString();
 
 	entry.flags = 0;
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = entry.ownerGroup;
 	entry.size = -1;
 
 	if (!line.GetToken(++index, token))
@@ -2423,7 +2400,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 	if (!line.GetToken(index++, token))
 		return false;
 
-	if (token.GetString().CmpNoCase(_T("Tape")))
+	wxString s = token.GetString();
+	MakeLowerAscii(s);
+	if (s != _T("tape"))
 		return false;
 
 	// dsname
@@ -2432,8 +2411,8 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 
 	entry.name = token.GetString();
 	entry.flags = 0;
-	entry.ownerGroup = objcache.get(wxString());
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(fzstring());
+	entry.permissions = objcache.get(fzstring());
 	entry.size = -1;
 
 	if (line.GetToken(index++, token))
@@ -2442,10 +2421,9 @@ bool CDirectoryListingParser::ParseAsIBM_MVS_Tape(CLine &line, CDirentry &entry)
 	return true;
 }
 
-bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, wxLongLong& size, int blocksize /*=-1*/)
+bool CDirectoryListingParser::ParseComplexFileSize(CToken& token, int64_t& size, int blocksize /*=-1*/)
 {
-	if (token.IsNumeric())
-	{
+	if (token.IsNumeric()) {
 		size = token.GetNumber();
 		if (blocksize != -1)
 			size *= blocksize;
@@ -2544,50 +2522,51 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 	if (!line.GetToken(0, token))
 		return 0;
 
-	wxString facts = token.GetString();
+	fzstring const facts = token.GetString();
 	if (facts.empty())
 		return 0;
 
 	entry.flags = 0;
 	entry.size = -1;
 
-	wxString owner, group, uid, gid;
-	wxString ownerGroup;
-	wxString permissions;
+	fzstring owner, group, uid, gid;
+	fzstring ownerGroup;
+	fzstring permissions;
 
-	while (!facts.empty()) {
-		int delim = facts.Find(';');
-		if (delim < 3) {
-			if (delim != -1)
-				return 0;
-			else
-				delim = facts.Len();
+	size_t start = 0;
+	while (start < facts.size()) {
+		auto delim = facts.find(';', start);
+		if (delim == fzstring::npos) {
+			delim = facts.size();
+		}
+		else if (delim < start + 3) {
+			return 0;
 		}
 
-		int const pos = facts.Find('=');
-		if (pos < 1 || pos > delim)
+		auto const pos = facts.find('=', start);
+		if (pos == fzstring::npos || pos < start + 1 || pos > delim)
 			return 0;
 
-		wxString factname = facts.Left(pos);
+		fzstring factname = facts.substr(start, pos - start);
 		MakeLowerAscii(factname);
-		wxString value = facts.Mid(pos + 1, delim - pos - 1);
+		fzstring value = facts.substr(pos + 1, delim - pos - 1);
 		if (factname == _T("type")) {
-			int colonPos = value.Find(':');
-			wxString valuePrefix;
-			if (colonPos == -1)
+			auto colonPos = value.find(':');
+			fzstring valuePrefix;
+			if (colonPos == fzstring::npos)
 				valuePrefix = value;
 			else
-				valuePrefix = value.Left(colonPos);
+				valuePrefix = value.substr(0, colonPos);
 			MakeLowerAscii(valuePrefix);
 
-			if (valuePrefix == _T("dir") && colonPos == -1)
+			if (valuePrefix == _T("dir") && colonPos == fzstring::npos)
 				entry.flags |= CDirentry::flag_dir;
 			else if (valuePrefix == _T("os.unix=slink") || valuePrefix == _T("os.unix=symlink")) {
 				entry.flags |= CDirentry::flag_dir | CDirentry::flag_link;
-				if (colonPos != -1)
-					entry.target = CSparseOptional<wxString>(value.Mid(colonPos));
+				if (colonPos != fzstring::npos)
+					entry.target = CSparseOptional<fzstring>(value.substr(colonPos));
 			}
-			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && colonPos == -1) {
+			else if ((valuePrefix == _T("cdir") || valuePrefix == _T("pdir")) && colonPos == fzstring::npos) {
 				// Current and parent directory, don't parse it
 				return 2;
 			}
@@ -2595,8 +2574,7 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 		else if (factname == _T("size")) {
 			entry.size = 0;
 
-			for (unsigned int i = 0; i < value.Len(); ++i)
-			{
+			for (unsigned int i = 0; i < value.size(); ++i) {
 				if (value[i] < '0' || value[i] > '9')
 					return 0;
 				entry.size *= 10;
@@ -2619,8 +2597,7 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 					permissions = value;
 			}
 		}
-		else if (factname == _T("unix.mode"))
-		{
+		else if (factname == _T("unix.mode")) {
 			if (!permissions.empty())
 				permissions = permissions + _T(" (") + value + _T(")");
 			else
@@ -2635,7 +2612,7 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 		else if (factname == _T("unix.gid"))
 			gid = value;
 
-		facts = facts.Mid(delim + 1);
+		start = delim + 1;
 	}
 
 	// The order of the facts is undefined, so assemble ownerGroup in correct
@@ -2649,13 +2626,12 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 	else if (!gid.empty())
 		ownerGroup += _T(" ") + gid;
 
-	entry.ownerGroup = objcache.get(ownerGroup);
-	entry.permissions = objcache.get(permissions);
-
 	if (!line.GetToken(1, token, true, true))
 		return 0;
 
 	entry.name = token.GetString();
+	entry.ownerGroup = objcache.get(ownerGroup);
+	entry.permissions = objcache.get(permissions);
 
 	return 1;
 }
@@ -2663,28 +2639,27 @@ int CDirectoryListingParser::ParseAsMlsd(CLine &line, CDirentry &entry)
 bool CDirectoryListingParser::ParseAsOS9(CLine &line, CDirentry &entry)
 {
 	int index = 0;
-	CToken token;
 
 	// Get owner
-	if (!line.GetToken(index++, token))
+	CToken ownerGroupToken;
+	if (!line.GetToken(index++, ownerGroupToken))
 		return false;
 
 	// Make sure it's number.number
-	int pos = token.Find('.');
-	if (pos == -1 || !pos || pos == ((int)token.GetLength() - 1))
+	int pos = ownerGroupToken.Find('.');
+	if (pos == -1 || !pos || pos == ((int)ownerGroupToken.GetLength() - 1))
 		return false;
 
-	if (!token.IsNumeric(0, pos))
+	if (!ownerGroupToken.IsNumeric(0, pos))
 		return false;
 
-	if (!token.IsNumeric(pos + 1, token.GetLength() - pos - 1))
+	if (!ownerGroupToken.IsNumeric(pos + 1, ownerGroupToken.GetLength() - pos - 1))
 		return false;
 
-	entry.ownerGroup = objcache.get(token.GetString());
-
 	entry.flags = 0;
 
 	// Get date
+	CToken token;
 	if (!line.GetToken(index++, token))
 		return false;
 
@@ -2696,12 +2671,11 @@ bool CDirectoryListingParser::ParseAsOS9(CLine &line, CDirentry &entry)
 		return false;
 
 	// Get perms
-	if (!line.GetToken(index++, token))
+	CToken permToken;
+	if (!line.GetToken(index++, permToken))
 		return false;
 
-	entry.permissions = objcache.get(token.GetString());
-
-	if (token[0] == 'd')
+	if (permToken[0] == 'd')
 		entry.flags |= CDirentry::flag_dir;
 
 	// Unused token
@@ -2722,6 +2696,8 @@ bool CDirectoryListingParser::ParseAsOS9(CLine &line, CDirentry &entry)
 		return false;
 
 	entry.name = token.GetString();
+	entry.ownerGroup = objcache.get(ownerGroupToken.GetString());
+	entry.permissions = objcache.get(permToken.GetString());
 
 	return true;
 }
@@ -2807,16 +2783,16 @@ bool CDirectoryListingParser::ParseAsZVM(CLine &line, CDirentry &entry)
 		return false;
 
 	// Owner
-	if (!line.GetToken(++index, token))
+	CToken ownerGroupToken;
+	if (!line.GetToken(++index, ownerGroupToken))
 		return false;
 
-	entry.ownerGroup = objcache.get(token.GetString());
-
 	// No further token!
 	if (line.GetToken(++index, token))
 		return false;
 
-	entry.permissions = objcache.get(wxString());
+	entry.ownerGroup = objcache.get(ownerGroupToken.GetString());
+	entry.permissions = objcache.get(fzstring());
 	entry.target.clear();
 	entry.time += m_timezoneOffset;
 
@@ -2865,26 +2841,27 @@ bool CDirectoryListingParser::ParseAsHPNonstop(CLine &line, CDirentry &entry)
 	// Owner
 	if (!line.GetToken(++index, token))
 		return false;
-	wxString ownerGroup = token.GetString();
+	fzstring ownerGroup = token.GetString();
 
-	if (token[token.GetLength() - 1] == ',')
-	{
+	if (token[token.GetLength() - 1] == ',') {
 		// Owner, part 2
 		if (!line.GetToken(++index, token))
 			return false;
 		ownerGroup += _T(" ") + token.GetString();
 	}
-	entry.ownerGroup = objcache.get(ownerGroup);
 
 	// Permissions
-	if (!line.GetToken(++index, token))
+	CToken permToken;
+	if (!line.GetToken(++index, permToken))
 		return false;
-	entry.permissions = objcache.get(token.GetString());
 
 	// Nothing
 	if (line.GetToken(++index, token))
 		return false;
 
+	entry.permissions = objcache.get(permToken.GetString());
+	entry.ownerGroup = objcache.get(ownerGroup);
+
 	return true;
 }
 
