@@ -5,7 +5,7 @@
 
 #include <gnutls/x509.h>
 
-#if GNUTLS_VERSION_NUMBER >= 0x030201
+#if FZ_USE_GNUTLS_SYSTEM_CIPHERS
 char const ciphers[] = "@SYSTEM";
 #else
 char const ciphers[] = "SECURE256:+SECURE128:-ARCFOUR-128:-3DES-CBC:-MD5:+SIGN-ALL:-SIGN-RSA-MD5:+CTYPE-X509:-CTYPE-OPENPGP:-VERS-SSL3.0";
@@ -25,14 +25,13 @@ void log_func(int level, const char* msg)
 }
 #endif
 
-CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
+CTlsSocket::CTlsSocket(CEventHandler* pEvtHandler, CSocket& socket, CControlSocket* pOwner)
 	: CEventHandler(pOwner->event_loop_)
 	, CBackend(pEvtHandler)
 	, m_pOwner(pOwner)
-	, m_pSocket(pSocket)
+	, m_socket(socket)
 {
-	wxASSERT(pSocket);
-	m_pSocketBackend = new CSocketBackend(this, *m_pSocket, m_pOwner->GetEngine().GetRateLimiter());
+	m_pSocketBackend = new CSocketBackend(this, m_socket, m_pOwner->GetEngine().GetRateLimiter());
 
 	m_implicitTrustedCert.data = 0;
 	m_implicitTrustedCert.size = 0;
@@ -311,7 +310,6 @@ void CTlsSocket::operator()(CEventBase const& ev)
 
 void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error)
 {
-	wxASSERT(m_pSocket);
 	if (!m_session)
 		return;
 
@@ -331,8 +329,7 @@ void CTlsSocket::OnSocketEvent(CSocketEventSource*, SocketEventType t, int error
 			if (peeked >= 0) {
 				if (peeked > 0)
 					m_pOwner->LogMessage(MessageType::Debug_Verbose, _T("CTlsSocket::OnSocketEvent(): pending data, postponing close event"));
-				else
-				{
+				else {
 					m_socket_eof = true;
 					m_socketClosed = true;
 				}
@@ -372,8 +369,7 @@ void CTlsSocket::OnRead()
 		ContinueHandshake();
 	if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == TlsState::conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -396,8 +392,7 @@ void CTlsSocket::OnSend()
 		ContinueHandshake();
 	else if (m_tlsState == TlsState::closing)
 		ContinueShutdown();
-	else if (m_tlsState == TlsState::conn)
-	{
+	else if (m_tlsState == TlsState::conn) {
 		CheckResumeFailedReadWrite();
 		TriggerEvents();
 	}
@@ -445,24 +440,22 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket, bool try_resume)
 		// Implicitly trust certificate of primary socket
 		unsigned int cert_list_size;
 		const gnutls_datum_t* const cert_list = gnutls_certificate_get_peers(pPrimarySocket->m_session, &cert_list_size);
-		if (cert_list && cert_list_size)
-		{
+		if (cert_list && cert_list_size) {
 			delete [] m_implicitTrustedCert.data;
 			m_implicitTrustedCert.data = new unsigned char[cert_list[0].size];
 			memcpy(m_implicitTrustedCert.data, cert_list[0].data, cert_list[0].size);
 			m_implicitTrustedCert.size = cert_list[0].size;
 		}
 
-		if (try_resume)
-		{
+		if (try_resume) {
 			if (!CopySessionData(pPrimarySocket))
 				return FZ_REPLY_ERROR;
 		}
 
-		hostname = pPrimarySocket->m_pSocket->GetPeerHost();
+		hostname = pPrimarySocket->m_socket.GetPeerHost();
 	}
 	else {
-		hostname = m_pSocket->GetPeerHost();
+		hostname = m_socket.GetPeerHost();
 	}
 
 	if( !hostname.empty() && !IsIpAddress(hostname) ) {
@@ -1013,6 +1006,61 @@ bool CTlsSocket::CertificateIsBlacklisted(std::vector<CCertificate> const& certi
 	return false;
 }
 
+
+int CTlsSocket::GetAlgorithmWarnings()
+{
+	int algorithmWarnings{};
+
+	switch (gnutls_protocol_get_version(m_session))
+	{
+		case GNUTLS_SSL3:
+		case GNUTLS_VERSION_UNKNOWN:
+			algorithmWarnings |= CCertificateNotification::tlsver;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_cipher_get(m_session)) {
+		case GNUTLS_CIPHER_UNKNOWN:
+		case GNUTLS_CIPHER_NULL:
+		case GNUTLS_CIPHER_ARCFOUR_128:
+		case GNUTLS_CIPHER_3DES_CBC:
+		case GNUTLS_CIPHER_ARCFOUR_40:
+		case GNUTLS_CIPHER_RC2_40_CBC:
+		case GNUTLS_CIPHER_DES_CBC:
+			algorithmWarnings |= CCertificateNotification::cipher;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_mac_get(m_session)) {
+		case GNUTLS_MAC_UNKNOWN:
+		case GNUTLS_MAC_NULL:
+		case GNUTLS_MAC_MD5:
+		case GNUTLS_MAC_MD2:
+		case GNUTLS_MAC_UMAC_96:
+			algorithmWarnings |= CCertificateNotification::mac;
+			break;
+		default:
+			break;
+	}
+
+	switch (gnutls_kx_get(m_session)) {
+		case GNUTLS_KX_UNKNOWN:
+		case GNUTLS_KX_ANON_DH:
+		case GNUTLS_KX_RSA_EXPORT:
+		case GNUTLS_KX_ANON_ECDH:
+			algorithmWarnings |= CCertificateNotification::kex;
+		default:
+			break;
+	}
+
+	return algorithmWarnings;
+}
+
+
 int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != TlsState::handshake) {
@@ -1093,6 +1141,8 @@ int CTlsSocket::VerifyCertificate()
 		return FZ_REPLY_ERROR;
 	}
 
+	int const algorithmWarnings = GetAlgorithmWarnings();
+
 	CCertificateNotification *pNotification = new CCertificateNotification(
 		m_pOwner->GetCurrentServer()->GetHost(),
 		m_pOwner->GetCurrentServer()->GetPort(),
@@ -1100,6 +1150,7 @@ int CTlsSocket::VerifyCertificate()
 		GetKeyExchange(),
 		GetCipherName(),
 		GetMacName(),
+		algorithmWarnings,
 		certificates);
 
 	m_pOwner->SendAsyncRequest(pNotification);
