@@ -256,24 +256,17 @@ CMainFrame::CMainFrame()
 	: m_engineContext(*COptions::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
 {
-#ifdef __WXMAC__
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Cut(); }, 'X');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Copy(); }, 'C');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->Paste(); }, 'V');
-	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e){ e->SelectAll(); }, 'A');
-#endif
-
 	m_pActivityLed[0] = m_pActivityLed[1] = 0;
 
 	wxGetApp().AddStartupProfileRecord(_T("CMainFrame::CMainFrame"));
 	wxRect screen_size = CWindowStateManager::GetScreenDimensions();
 
 	wxSize initial_size;
-	initial_size.x = wxMin(900, screen_size.GetWidth() - 10);
-	initial_size.y = wxMin(750, screen_size.GetHeight() - 50);
+	initial_size.x = wxMin(1200, screen_size.GetWidth() - 10);
+	initial_size.y = wxMin(950, screen_size.GetHeight() - 50);
 
 	Create(NULL, -1, _T("FileZilla"), wxDefaultPosition, initial_size);
-	SetSizeHints(250, 250);
+	SetSizeHints(350, 300);
 
 #ifdef __WXMSW__
 	// In order for the --close commandline argument to work,
@@ -338,7 +331,7 @@ CMainFrame::CMainFrame()
 	m_pTopSplitter->SetMinimumPaneSize(50);
 
 	m_pBottomSplitter = new CSplitterWindowEx(m_pTopSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER | wxSP_LIVE_UPDATE);
-	m_pBottomSplitter->SetMinimumPaneSize(10, 60);
+	m_pBottomSplitter->SetMinimumPaneSize(20, 70);
 	m_pBottomSplitter->SetSashGravity(1.0);
 
 	const int message_log_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
@@ -425,20 +418,7 @@ CMainFrame::CMainFrame()
 	if (!RestoreSplitterPositions())
 		SetDefaultSplitterPositions();
 
-	std::vector<wxAcceleratorEntry> entries;
-	//entries[0].Set(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
-	for (int i = 0; i < 10; i++) {
-		tab_hotkey_ids[i] = wxNewId();
-		entries.push_back(wxAcceleratorEntry(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]));
-	}
-	entries.push_back(wxAcceleratorEntry(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId));
-#ifdef __WXMAC__
-	for (auto it = keyboardCommands.begin(); it != keyboardCommands.end(); ++it) {
-		entries.push_back(wxAcceleratorEntry(wxACCEL_CMD, it->second.second, it->first));
-	}
-#endif
-	wxAcceleratorTable accel(entries.size(), &entries[0]);
-	SetAcceleratorTable(accel);
+	SetupKeyboardAccelerators();
 
 	ConnectNavigationHandler(m_pStatusView);
 	ConnectNavigationHandler(m_pQueuePane);
@@ -448,10 +428,19 @@ CMainFrame::CMainFrame()
 	CAutoAsciiFiles::SettingsChanged();
 
 	FixTabOrder();
+
+	RegisterOption(OPTION_LANGUAGE);
+	RegisterOption(OPTION_THEME);
+	RegisterOption(OPTION_THEME_ICONSIZE);
+	RegisterOption(OPTION_MESSAGELOG_POSITION);
+	RegisterOption(OPTION_FILEPANE_LAYOUT);
+	RegisterOption(OPTION_FILEPANE_SWAP);
 }
 
 CMainFrame::~CMainFrame()
 {
+	UnregisterAllOptions();
+
 	CPowerManagement::Destroy();
 
 	delete m_pStateEventHandler;
@@ -1280,8 +1269,7 @@ void CMainFrame::OnReconnect(wxCommandEvent &)
 
 	CServer server = pState->GetLastServer();
 
-	if (server.GetLogonType() == ASK)
-	{
+	if (server.GetLogonType() == ASK) {
 		if (!CLoginManager::Get().GetPassword(server, false))
 			return;
 	}
@@ -1432,42 +1420,17 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 
 	COptions* pOptions = COptions::Get();
 
-	wxString oldTheme = pOptions->GetOption(OPTION_THEME);
-	wxString oldThemeSize = pOptions->GetOption(OPTION_THEME_ICONSIZE);
 	wxString oldLang = pOptions->GetOption(OPTION_LANGUAGE);
 
 	int oldShowDebugMenu = pOptions->GetOptionVal(OPTION_DEBUG_MENU) != 0;
 
-	bool oldTimestamps = pOptions->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-
 	int res = dlg.ShowModal();
-	if (res != wxID_OK)
-	{
-		UpdateLayout();
+	if (res != wxID_OK) {
 		return;
 	}
 
-	bool newTimestamps = pOptions->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
-
-	wxString newTheme = pOptions->GetOption(OPTION_THEME);
-	wxString newThemeSize = pOptions->GetOption(OPTION_THEME_ICONSIZE);
 	wxString newLang = pOptions->GetOption(OPTION_LANGUAGE);
 
-	if (oldTheme != newTheme ||
-		oldThemeSize != newThemeSize ||
-		oldLang != newLang)
-	{
-		CreateMainToolBar();
-		if (m_pToolBar)
-			m_pToolBar->UpdateToolbarState();
-	}
-
-	if (oldLang != newLang ||
-		oldTimestamps != newTimestamps)
-	{
-		m_pStatusView->InitDefAttr();
-	}
-
 	if (oldLang != newLang ||
 		oldShowDebugMenu != pOptions->GetOptionVal(OPTION_DEBUG_MENU))
 	{
@@ -1487,42 +1450,34 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 
 	bool shown;
 
-	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1)
-	{
+	if (COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter)
 			return;
-		if (m_pQueueLogSplitter->IsSplit())
-		{
+		if (m_pQueueLogSplitter->IsSplit()) {
 			m_pQueueLogSplitter->Unsplit(m_pStatusView);
 			shown = false;
 		}
-		else if (m_pStatusView->IsShown())
-		{
+		else if (m_pStatusView->IsShown()) {
 			m_pStatusView->Hide();
 			m_pBottomSplitter->Unsplit(m_pQueueLogSplitter);
 			shown = false;
 		}
-		else if (!m_pQueueLogSplitter->IsShown())
-		{
+		else if (!m_pQueueLogSplitter->IsShown()) {
 			m_pQueueLogSplitter->Initialize(m_pStatusView);
 			m_pBottomSplitter->SplitHorizontally(m_pContextControl, m_pQueueLogSplitter);
 			shown = true;
 		}
-		else
-		{
+		else {
 			m_pQueueLogSplitter->SplitVertically(m_pQueuePane, m_pStatusView);
 			shown = true;
 		}
 	}
-	else
-	{
-		if (m_pTopSplitter->IsSplit())
-		{
+	else {
+		if (m_pTopSplitter->IsSplit()) {
 			m_pTopSplitter->Unsplit(m_pStatusView);
 			shown = false;
 		}
-		else
-		{
+		else {
 			m_pTopSplitter->SplitHorizontally(m_pStatusView, m_pBottomSplitter);
 			shown = true;
 		}
@@ -1736,15 +1691,12 @@ void CMainFrame::TriggerUpdateDialog()
 }
 #endif
 
-void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagelog_position /*=-1*/)
+void CMainFrame::UpdateLayout()
 {
-	if (layout == -1)
-		layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
-	if (swap == -1)
-		swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
+	int const layout = COptions::Get()->GetOptionVal(OPTION_FILEPANE_LAYOUT);
+	int const swap = COptions::Get()->GetOptionVal(OPTION_FILEPANE_SWAP);
 
-	if (messagelog_position == -1)
-		messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
+	int const messagelog_position = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_POSITION);
 
 	// First handle changes in message log position as it can make size of the other panes change
 	{
@@ -1752,16 +1704,13 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 		wxWindow* parent = m_pStatusView->GetParent();
 
 		bool changed;
-		if (parent == m_pTopSplitter && messagelog_position != 0)
-		{
+		if (parent == m_pTopSplitter && messagelog_position != 0) {
 			if (shown)
 				m_pTopSplitter->Unsplit(m_pStatusView);
 			changed = true;
 		}
-		else if (parent == m_pQueueLogSplitter && messagelog_position != 1)
-		{
-			if (shown)
-			{
+		else if (parent == m_pQueueLogSplitter && messagelog_position != 1) {
+			if (shown) {
 				if (m_pQueueLogSplitter->IsSplit())
 					m_pQueueLogSplitter->Unsplit(m_pStatusView);
 				else
@@ -1769,8 +1718,7 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 			}
 			changed = true;
 		}
-		else if (parent != m_pTopSplitter && parent != m_pQueueLogSplitter && messagelog_position != 2)
-		{
+		else if (parent != m_pTopSplitter && parent != m_pQueueLogSplitter && messagelog_position != 2) {
 			m_pQueuePane->RemovePage(3);
 			changed = true;
 			shown = true;
@@ -1778,10 +1726,8 @@ void CMainFrame::UpdateLayout(int layout /*=-1*/, int swap /*=-1*/, int messagel
 		else
 			changed = false;
 
-		if (changed)
-		{
-			switch (messagelog_position)
-			{
+		if (changed) {
+			switch (messagelog_position) {
 			default:
 				m_pStatusView->Reparent(m_pTopSplitter);
 				if (shown)
@@ -1975,8 +1921,6 @@ bool CMainFrame::ConnectToSite(CSiteManagerItemData_Site & data, bool newTab)
 
 void CMainFrame::CheckChangedSettings()
 {
-	UpdateLayout();
-
 	m_pAsyncRequestQueue->RecheckDefaults();
 
 	CAutoAsciiFiles::SettingsChanged();
@@ -2372,14 +2316,17 @@ void CMainFrame::ProcessCommandLine()
 			server.SetLogonType(INTERACTIVE);
 
 		CServerPath path;
-		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path))
-		{
+		if (!server.ParseUrl(param, 0, _T(""), _T(""), error, path)) {
 			wxString str = _("Parameter not a valid URL");
 			str += _T("\n") + error;
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (server.GetLogonType() == ASK ||
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) && server.GetLogonType() == NORMAL) {
+			server.SetLogonType(ASK);
+			CLoginManager::Get().RememberPassword(server);
+		}
+		else if (server.GetLogonType() == ASK ||
 			(server.GetLogonType() == INTERACTIVE && server.GetUser().empty()))
 		{
 			if (!CLoginManager::Get().GetPassword(server, false))
@@ -2540,6 +2487,11 @@ void CMainFrame::OnSearch(wxCommandEvent& event)
 	if (!pState)
 		return;
 
+	if (!pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
+		wxBell();
+		return;
+	}
+
 	CSearchDialog dlg(this, pState, m_pQueueView);
 	if (!dlg.Load())
 		return;
@@ -2689,3 +2641,43 @@ void CMainFrame::OnChildFocused(wxChildFocusEvent& event)
 	m_lastFocusedChild = event.GetWindow()->GetId();
 }
 #endif
+
+void CMainFrame::SetupKeyboardAccelerators()
+{
+	std::vector<wxAcceleratorEntry> entries;
+	for (int i = 0; i < 10; i++) {
+		tab_hotkey_ids[i] = wxNewId();
+		entries.emplace_back(wxACCEL_CMD, (int)'0' + i, tab_hotkey_ids[i]);
+	}
+	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'O', m_comparisonToggleAcceleratorId);
+	entries.emplace_back(wxACCEL_CMD | wxACCEL_SHIFT, 'I', XRCID("ID_MENU_VIEW_FILTERS"));
+#ifdef __WXMAC__
+	entries.emplace_back(wxACCEL_CMD, ',', XRCID("wxID_PREFERENCES"));
+
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Cut(); }, 'X');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Copy(); }, 'C');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->Paste(); }, 'V');
+	keyboardCommands[wxNewId()] = std::make_pair([](wxTextEntry* e) { e->SelectAll(); }, 'A');
+
+	for (auto const& command : keyboardCommands) {
+		entries.emplace_back(wxACCEL_CMD, command.second.second, command.first);
+	}
+#endif
+	wxAcceleratorTable accel(entries.size(), &entries[0]);
+	SetAcceleratorTable(accel);
+}
+
+void CMainFrame::OnOptionsChanged(changed_options_t const& options)
+{
+	if (options.test(OPTION_FILEPANE_LAYOUT) || options.test(OPTION_FILEPANE_SWAP) || options.test(OPTION_MESSAGELOG_POSITION)) {
+		UpdateLayout();
+	}
+
+	bool const language_changed = options.test(OPTION_LANGUAGE);
+	if (options.test(OPTION_THEME) || options.test(OPTION_THEME_ICONSIZE) || language_changed) {
+		CreateMainToolBar();
+		if (m_pToolBar) {
+			m_pToolBar->UpdateToolbarState();
+		}
+	}
+}
