@@ -26,82 +26,42 @@ void CQueueItem::SetPriority(QueuePriority priority)
 
 void CQueueItem::AddChild(CQueueItem* item)
 {
+	wxASSERT(GetType() != QueueItemType::Folder);
+	wxASSERT(GetType() != QueueItemType::Status);
+
 	if (m_removed_at_front) {
 		m_children.erase(m_children.begin(), m_children.begin() + m_removed_at_front);
 		m_removed_at_front = 0;
 	}
 	m_children.push_back(item);
 
-	m_visibleOffspring += 1 + item->m_visibleOffspring;
-	m_maxCachedIndex = -1;
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_visibleOffspring += 1 + item->m_visibleOffspring;
-		parent->m_maxCachedIndex = -1;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_visibleOffspring += 1 + item->GetChildrenCount(true);
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+		}
 		parent = parent->GetParent();
 	}
 }
 
-CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
+
+CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive)
 {
-	std::vector<CQueueItem*>::iterator iter;
-	if (!recursive)
-	{
+	if (!recursive) {
 		if (item + m_removed_at_front >= m_children.size())
 			return 0;
-		iter = m_children.begin();
-		iter += item + m_removed_at_front;
-		return *iter;
+		return *(m_children.begin() + item + m_removed_at_front);
 	}
 
-	if ((int)item <= m_maxCachedIndex)
-	{
-		// Index is cached
-		iter = m_children.begin() + m_removed_at_front;
-		iter += m_lookupCache[item].child;
-		item -= m_lookupCache[item].index;
-		if (!item)
-			return *iter;
-		else
-			return (*iter)->GetChild(item - 1);
-	}
-
-	int index;
-	int child;
-	iter = m_children.begin() + m_removed_at_front;
-	if (m_maxCachedIndex == -1)
-	{
-		child = 0;
-		index = 0;
-	}
-	else
-	{
-		// Start with loop with the last cached item index
-		iter += m_lookupCache[m_maxCachedIndex].child + 1;
-		item -= m_maxCachedIndex + 1;
-		index = m_maxCachedIndex + 1;
-		child = m_lookupCache[m_maxCachedIndex].child + 1;
-	}
-
-	for (; iter != m_children.end(); ++iter, ++child)
-	{
+	std::vector<CQueueItem*>::iterator iter = m_children.begin() + m_removed_at_front;
+	for (; iter != m_children.end(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
-			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
-				m_lookupCache.resize(m_visibleOffspring);
-			for (unsigned int k = index; k <= index + count; k++)
-			{
-				m_lookupCache[k].child = child;
-				m_lookupCache[k].index = index;
-			}
-			m_maxCachedIndex = index + count;
+		if (item > count) {
 			item -= count + 1;
-			index += count + 1;
 			continue;
 		}
 
@@ -110,29 +70,36 @@ CQueueItem* CQueueItem::GetChild(unsigned int item, bool recursive /*=true*/)
 	return 0;
 }
 
-unsigned int CQueueItem::GetChildrenCount(bool recursive)
+unsigned int CQueueItem::GetChildrenCount(bool recursive) const
 {
-	if (!recursive)
-		return m_children.size() - m_removed_at_front;
+	unsigned int count{};
+	if (!recursive) {
+		count = m_children.size() - m_removed_at_front;
+	}
+	else {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
+			count += 1 + (*iter)->GetChildrenCount(true);
+		}
+	}
 
-	return m_visibleOffspring;
+	return count;
 }
 
-bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
-	int oldVisibleOffspring = m_visibleOffspring;
+	int const oldVisibleOffspring = GetChildrenCount(true);
+	int visibleOffspring = oldVisibleOffspring;
+
 	bool deleted = false;
-	for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		if (*iter == pItem)
-		{
-			m_visibleOffspring -= 1;
-			m_visibleOffspring -= pItem->m_visibleOffspring;
+
+	auto doRemove = [&](std::vector<CQueueItem*>::iterator iter) {
+		if (*iter == pItem) {
+			visibleOffspring -= 1;
+			visibleOffspring -= pItem->GetChildrenCount(true);
 			if (destroy)
 				delete pItem;
 
-			if (iter - m_children.begin() - m_removed_at_front <= 10)
-			{
+			if (iter - m_children.begin() - m_removed_at_front <= 10) {
 				m_removed_at_front++;
 				unsigned int end = iter - m_children.begin();
 				int c = 0;
@@ -143,20 +110,17 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 				m_children.erase(iter);
 
 			deleted = true;
-			break;
+			return;
 		}
 
-		int visibleOffspring = (*iter)->m_visibleOffspring;
-		if ((*iter)->RemoveChild(pItem, destroy))
-		{
-			m_visibleOffspring -= visibleOffspring - (*iter)->m_visibleOffspring;
-			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front))
-			{
-				m_visibleOffspring -= 1;
+		int childVisibleOffspring = (*iter)->GetChildrenCount(true);
+		if ((*iter)->RemoveChild(pItem, destroy)) {
+			visibleOffspring -= childVisibleOffspring - (*iter)->GetChildrenCount(true);
+			if (!((*iter)->m_children.size() - (*iter)->m_removed_at_front)) {
+				visibleOffspring -= 1;
 				delete *iter;
 
-				if (iter - m_children.begin() - m_removed_at_front <= 10)
-				{
+				if (iter - m_children.begin() - m_removed_at_front <= 10) {
 					m_removed_at_front++;
 					unsigned int end = iter - m_children.begin();
 					for (int i = end; i >= m_removed_at_front; i--)
@@ -167,60 +131,40 @@ bool CQueueItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
 			}
 
 			deleted = true;
-			break;
+		}
+	};
+
+	if (forward) {
+		for (auto iter = m_children.begin() + m_removed_at_front; iter != m_children.end() && !deleted; ++iter) {
+			doRemove(iter);
+		}
+	}
+	else {
+		for (auto iter = m_children.rbegin(); iter != m_children.rend() - m_removed_at_front && !deleted; ++iter) {
+			doRemove(iter.base() - 1);
 		}
 	}
 
 	if (!deleted)
 		return false;
 
-	m_maxCachedIndex = -1;
+	if (GetType() == QueueItemType::Server) {
+		static_cast<CServerItem*>(this)->m_visibleOffspring = visibleOffspring;
+	}
 
 	// Propagate new children count to parent
 	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
+	while (parent) {
+		if (parent->GetType() == QueueItemType::Server) {
+			static_cast<CServerItem*>(parent)->m_maxCachedIndex = -1;
+			static_cast<CServerItem*>(parent)->m_visibleOffspring -= oldVisibleOffspring - visibleOffspring;
+		}
 		parent = parent->GetParent();
 	}
 
 	return true;
 }
 
-bool CQueueItem::TryRemoveAll()
-{
-	const int oldVisibleOffspring = m_visibleOffspring;
-	std::vector<CQueueItem*>::iterator iter;
-	std::vector<CQueueItem*> keepChildren;
-	m_visibleOffspring = 0;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
-		CQueueItem* pItem = *iter;
-		if (pItem->TryRemoveAll())
-			delete pItem;
-		else
-		{
-			keepChildren.push_back(pItem);
-			m_visibleOffspring++;
-			m_visibleOffspring += pItem->GetChildrenCount(true);
-		}
-	}
-	m_children = keepChildren;
-	m_maxCachedIndex = -1;
-	m_removed_at_front = 0;
-
-	CQueueItem* parent = GetParent();
-	while (parent)
-	{
-		parent->m_maxCachedIndex = -1;
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
-
-	return m_children.empty();
-}
-
 CQueueItem* CQueueItem::GetTopLevelItem()
 {
 	if (!m_parent)
@@ -228,8 +172,7 @@ CQueueItem* CQueueItem::GetTopLevelItem()
 
 	CQueueItem* newParent = m_parent;
 	CQueueItem* parent = 0;
-	while (newParent)
-	{
+	while (newParent) {
 		parent = newParent;
 		newParent = newParent->GetParent();
 	}
@@ -276,7 +219,7 @@ CFileItem::CFileItem(CServerItem* parent, bool queued, bool download,
 					 const CLocalPath& localPath, const CServerPath& remotePath, int64_t size)
 	: CQueueItem(parent)
 	, m_sourceFile(sourceFile)
-	, m_targetFile(targetFile.empty() ? CSparseOptional<wxString>() : CSparseOptional<wxString>(targetFile))
+	, m_targetFile(targetFile.empty() ? CSparseOptional<fzstring>() : CSparseOptional<fzstring>(to_fzstring(targetFile)))
 	, m_localPath(localPath)
 	, m_remotePath(remotePath)
 	, m_size(size)
@@ -317,6 +260,7 @@ QueuePriority CFileItem::GetPriority() const
 void CFileItem::SetActive(const bool active)
 {
 	if (active && !IsActive()) {
+		wxASSERT(!GetChildrenCount(false));
 		AddChild(new CStatusItem);
 		flags |= flag_active;
 	}
@@ -327,19 +271,19 @@ void CFileItem::SetActive(const bool active)
 	}
 }
 
-void CFileItem::SaveItem(TiXmlElement* pElement) const
+void CFileItem::SaveItem(pugi::xml_node& element) const
 {
-	if (m_edit != CEditHandler::none)
+	if (m_edit != CEditHandler::none || !element)
 		return;
 
-	TiXmlElement *file = pElement->LinkEndChild(new TiXmlElement("File"))->ToElement();
+	auto file = element.append_child("File");
 
 	AddTextElement(file, "LocalFile", m_localPath.GetPath() + GetLocalFile());
 	AddTextElement(file, "RemoteFile", GetRemoteFile());
 	AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
 	if (m_size != -1)
-		AddTextElement(file, "Size", wxLongLong(m_size).ToString());
+		AddTextElement(file, "Size", m_size);
 	if (m_errorCount)
 		AddTextElement(file, "ErrorCount", m_errorCount);
 	if (m_priority != QueuePriority::normal)
@@ -361,7 +305,7 @@ bool CFileItem::TryRemoveAll()
 void CFileItem::SetTargetFile(wxString const& file)
 {
 	if (!file.empty() && file != m_sourceFile)
-		m_targetFile = CSparseOptional<wxString>(file);
+		m_targetFile = CSparseOptional<fzstring>(to_fzstring(file));
 	else
 		m_targetFile.clear();
 }
@@ -403,20 +347,17 @@ CFolderItem::CFolderItem(CServerItem* parent, bool queued, const CServerPath& re
 {
 }
 
-void CFolderItem::SaveItem(TiXmlElement* pElement) const
+void CFolderItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *file = new TiXmlElement("Folder");
+	auto file = element.append_child("Folder");
 
 	if (Download())
 		AddTextElement(file, "LocalFile", GetLocalPath().GetPath() + GetLocalFile());
-	else
-	{
+	else {
 		AddTextElement(file, "RemoteFile", GetRemoteFile());
 		AddTextElement(file, "RemotePath", m_remotePath.GetSafePath());
 	}
 	AddTextElementRaw(file, "Download", Download() ? "1" : "0");
-
-	pElement->LinkEndChild(file);
 }
 
 void CFolderItem::SetActive(const bool active)
@@ -450,9 +391,22 @@ wxString CServerItem::GetName() const
 void CServerItem::AddChild(CQueueItem* pItem)
 {
 	CQueueItem::AddChild(pItem);
+	m_maxCachedIndex = -1;
+	m_visibleOffspring += 1 + pItem->GetChildrenCount(true);
 	if (pItem->GetType() == QueueItemType::File ||
 		pItem->GetType() == QueueItemType::Folder)
 		AddFileItemToList((CFileItem*)pItem);
+
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+}
+
+unsigned int CServerItem::GetChildrenCount(bool recursive) const
+{
+	if (!recursive)
+		return m_children.size() - m_removed_at_front;
+
+	return m_visibleOffspring;
 }
 
 void CServerItem::AddFileItemToList(CFileItem* pItem)
@@ -463,13 +417,23 @@ void CServerItem::AddFileItemToList(CFileItem* pItem)
 	m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())].push_back(pItem);
 }
 
-void CServerItem::RemoveFileItemFromList(CFileItem* pItem)
+void CServerItem::RemoveFileItemFromList(CFileItem* pItem, bool forward)
 {
-	std::list<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
-		if (*iter == pItem) {
-			fileList.erase(iter);
-			return;
+	std::deque<CFileItem*>& fileList = m_fileList[pItem->queued() ? 0 : 1][static_cast<int>(pItem->GetPriority())];
+	if (forward) {
+		for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter);
+				return;
+			}
+		}
+	}
+	else {
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+			if (*iter == pItem) {
+				fileList.erase(iter.base() - 1);
+				return;
+			}
 		}
 	}
 	wxFAIL_MSG(_T("File item not deleted from m_fileList"));
@@ -495,8 +459,68 @@ void CServerItem::SetDefaultFileExistsAction(CFileExistsNotification::OverwriteA
 	}
 }
 
+CQueueItem* CServerItem::GetChild(unsigned int item, bool recursive)
+{
+	std::vector<CQueueItem*>::iterator iter;
+	if (!recursive) {
+		if (item + m_removed_at_front >= m_children.size())
+			return 0;
+		iter = m_children.begin();
+		iter += item + m_removed_at_front;
+		return *iter;
+	}
+
+	if ((int)item <= m_maxCachedIndex) {
+		// Index is cached
+		iter = m_children.begin() + m_removed_at_front;
+		iter += m_lookupCache[item].child;
+		item -= m_lookupCache[item].index;
+		if (!item)
+			return *iter;
+		else
+			return (*iter)->GetChild(item - 1);
+	}
+
+	int index;
+	int child;
+	iter = m_children.begin() + m_removed_at_front;
+	if (m_maxCachedIndex == -1) {
+		child = 0;
+		index = 0;
+	}
+	else {
+		// Start with loop with the last cached item index
+		iter += m_lookupCache[m_maxCachedIndex].child + 1;
+		item -= m_maxCachedIndex + 1;
+		index = m_maxCachedIndex + 1;
+		child = m_lookupCache[m_maxCachedIndex].child + 1;
+	}
+
+	for (; iter != m_children.end(); ++iter, ++child) {
+		if (!item)
+			return *iter;
+
+		unsigned int count = (*iter)->GetChildrenCount(true);
+		if (item > count) {
+			if (m_maxCachedIndex == -1 && m_lookupCache.size() < (unsigned int)m_visibleOffspring)
+				m_lookupCache.resize(m_visibleOffspring);
+			for (unsigned int k = index; k <= index + count; ++k) {
+				m_lookupCache[k].child = child;
+				m_lookupCache[k].index = index;
+			}
+			m_maxCachedIndex = index + count;
+			item -= count + 1;
+			index += count + 1;
+			continue;
+		}
+
+		return (*iter)->GetChild(item - 1);
+	}
+	return 0;
+}
+
 namespace {
-CFileItem* DoGetIdleChild(std::list<CFileItem*> const* fileList, TransferDirection direction)
+CFileItem* DoGetIdleChild(std::deque<CFileItem*> const* fileList, TransferDirection direction)
 {
 	int i = 0;
 	for (i = static_cast<int>(QueuePriority::count) - 1; i >= 0; --i) {
@@ -529,37 +553,43 @@ CFileItem* CServerItem::GetIdleChild(bool immediateOnly, TransferDirection direc
 	return item;
 }
 
-bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy /*=true*/)
+bool CServerItem::RemoveChild(CQueueItem* pItem, bool destroy, bool forward)
 {
 	if (!pItem)
 		return false;
 
-	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder)
-	{
+	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
-		RemoveFileItemFromList(pFileItem);
+		RemoveFileItemFromList(pFileItem, forward);
 	}
 
-	return CQueueItem::RemoveChild(pItem, destroy);
+	bool removed = CQueueItem::RemoveChild(pItem, destroy, forward);
+	if (removed) {
+		m_maxCachedIndex = -1;
+	}
+	
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()) - m_removed_at_front);
+	wxASSERT(((m_children.size() - m_removed_at_front) != 0) == (m_visibleOffspring != 0));
+
+	return removed;
 }
 
 void CServerItem::QueueImmediateFiles()
 {
 	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
-		std::list<CFileItem*> activeList;
-		std::list<CFileItem*>& fileList = m_fileList[1][i];
-		for (std::list<CFileItem*>::reverse_iterator iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
+		std::deque<CFileItem*> activeList;
+		std::deque<CFileItem*>& fileList = m_fileList[1][i];
+		for (auto iter = fileList.rbegin(); iter != fileList.rend(); ++iter) {
 			CFileItem* item = *iter;
 			wxASSERT(!item->queued());
 			if (item->IsActive())
 				activeList.push_front(item);
-			else
-			{
+			else {
 				item->set_queued(true);
 				m_fileList[0][i].push_front(item);
 			}
 		}
-		fileList = activeList;
+		std::swap(fileList, activeList);
 	}
 }
 
@@ -568,9 +598,8 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	if (pItem->queued())
 		return;
 
-	std::list<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
-	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter)
-	{
+	std::deque<CFileItem*>& fileList = m_fileList[1][static_cast<int>(pItem->GetPriority())];
+	for (auto iter = fileList.begin(); iter != fileList.end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -582,29 +611,23 @@ void CServerItem::QueueImmediateFile(CFileItem* pItem)
 	wxASSERT(false);
 }
 
-void CServerItem::SaveItem(TiXmlElement* pElement) const
+void CServerItem::SaveItem(pugi::xml_node& element) const
 {
-	TiXmlElement *server = new TiXmlElement("Server");
+	auto server = element.append_child("Server");
 	SetServer(server, m_server);
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
+	for (auto iter = m_children.cbegin() + m_removed_at_front; iter != m_children.cend(); ++iter)
 		(*iter)->SaveItem(server);
-
-	pElement->LinkEndChild(server);
 }
 
-wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
+int64_t CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles, int& folderScanCount) const
 {
-	wxLongLong totalSize = 0;
-	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i)
-	{
-		for (int j = 0; j < 2; j++)
-		{
-			const std::list<CFileItem*>& fileList = m_fileList[j][i];
-			for (std::list<CFileItem*>::const_iterator iter = fileList.begin(); iter != fileList.end(); ++iter)
-			{
-				const CFileItem* item = *iter;
-				wxLongLong size = item->GetSize();
+	int64_t totalSize = 0;
+	for (int i = 0; i < static_cast<int>(QueuePriority::count); ++i) {
+		for (int j = 0; j < 2; ++j) {
+			const std::deque<CFileItem*>& fileList = m_fileList[j][i];
+			for (auto const& item : fileList) {
+				int64_t size = item->GetSize();
 				if (size >= 0)
 					totalSize += size;
 				else
@@ -613,8 +636,7 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 		}
 	}
 
-	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (std::vector<CQueueItem*>::const_iterator iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File ||
 			(*iter)->GetType() == QueueItemType::Folder)
 			queuedFiles++;
@@ -627,6 +649,8 @@ wxLongLong CServerItem::GetTotalSize(int& filesWithUnknownSize, int& queuedFiles
 
 bool CServerItem::TryRemoveAll()
 {
+	wxASSERT(!GetParent());
+
 	const int oldVisibleOffspring = m_visibleOffspring;
 	std::vector<CQueueItem*>::iterator iter;
 	std::vector<CQueueItem*> keepChildren;
@@ -636,7 +660,7 @@ bool CServerItem::TryRemoveAll()
 		if (pItem->TryRemoveAll()) {
 			if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 				CFileItem* pFileItem = static_cast<CFileItem*>(pItem);
-				RemoveFileItemFromList(pFileItem);
+				RemoveFileItemFromList(pFileItem, true);
 			}
 			delete pItem;
 		}
@@ -646,14 +670,14 @@ bool CServerItem::TryRemoveAll()
 			m_visibleOffspring += pItem->GetChildrenCount(true);
 		}
 	}
-	m_children = keepChildren;
+	std::swap(m_children, keepChildren);
 	m_removed_at_front = 0;
 
-	CQueueItem* parent = GetParent();
-	while (parent) {
-		parent->m_visibleOffspring -= oldVisibleOffspring - m_visibleOffspring;
-		parent = parent->GetParent();
-	}
+	m_maxCachedIndex = -1;
+
+	wxASSERT(oldVisibleOffspring >= m_visibleOffspring);
+	wxASSERT(m_visibleOffspring >= static_cast<int>(m_children.size()));
+	(void)oldVisibleOffspring;
 
 	return m_children.empty();
 }
@@ -667,7 +691,7 @@ void CServerItem::DetachChildren()
 	m_maxCachedIndex = -1;
 	m_removed_at_front = 0;
 
-	for (int i = 0; i < 2; i++)
+	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); j++)
 			m_fileList[i][j].clear();
 }
@@ -675,8 +699,7 @@ void CServerItem::DetachChildren()
 void CServerItem::SetPriority(QueuePriority priority)
 {
 	std::vector<CQueueItem*>::iterator iter;
-	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter)
-	{
+	for (iter = m_children.begin() + m_removed_at_front; iter != m_children.end(); ++iter) {
 		if ((*iter)->GetType() == QueueItemType::File)
 			((CFileItem*)(*iter))->SetPriorityRaw(priority);
 		else
@@ -685,8 +708,10 @@ void CServerItem::SetPriority(QueuePriority priority)
 
 	for (int i = 0; i < 2; ++i)
 		for (int j = 0; j < static_cast<int>(QueuePriority::count); ++j) {
-			if (j != static_cast<int>(priority))
-				m_fileList[i][static_cast<int>(priority)].splice(m_fileList[i][static_cast<int>(priority)].end(), m_fileList[i][j]);
+			if (j != static_cast<int>(priority)) {
+				std::move(m_fileList[i][j].begin(), m_fileList[i][j].end(), std::back_inserter(m_fileList[i][static_cast<int>(priority)]));
+				m_fileList[i][j].clear();
+			}
 		}
 }
 
@@ -694,8 +719,7 @@ void CServerItem::SetChildPriority(CFileItem* pItem, QueuePriority oldPriority,
 {
 	int i = pItem->queued() ? 0 : 1;
 
-	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter)
-	{
+	for (auto iter = m_fileList[i][static_cast<int>(oldPriority)].begin(); iter != m_fileList[i][static_cast<int>(oldPriority)].end(); ++iter) {
 		if (*iter != pItem)
 			continue;
 
@@ -735,6 +759,7 @@ EVT_CHAR(CQueueViewBase::OnChar)
 EVT_LIST_COL_END_DRAG(wxID_ANY, CQueueViewBase::OnEndColumnDrag)
 EVT_TIMER(wxID_ANY, CQueueViewBase::OnTimer)
 EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
+EVT_MENU(XRCID("ID_EXPORT"), CQueueViewBase::OnExport)
 END_EVENT_TABLE()
 
 CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
@@ -773,17 +798,14 @@ CQueueViewBase::~CQueueViewBase()
 		delete *iter;
 }
 
-CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item)
+CQueueItem* CQueueViewBase::GetQueueItem(unsigned int item) const
 {
-	std::vector<CServerItem*>::iterator iter;
-	for (iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
-	{
+	for (auto iter = m_serverList.cbegin(); iter != m_serverList.cend(); ++iter) {
 		if (!item)
 			return *iter;
 
 		unsigned int count = (*iter)->GetChildrenCount(true);
-		if (item > count)
-		{
+		if (item > count) {
 			item -= count + 1;
 			continue;
 		}
@@ -1298,7 +1320,7 @@ void CQueueViewBase::InsertItem(CServerItem* pServerItem, CQueueItem* pItem)
 	}
 }
 
-bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount /*=true*/, bool updateSelections /*=true*/)
+bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItemCount, bool updateSelections, bool forward)
 {
 	if (pItem->GetType() == QueueItemType::File || pItem->GetType() == QueueItemType::Folder) {
 		wxASSERT(m_fileCount > 0);
@@ -1319,7 +1341,7 @@ bool CQueueViewBase::RemoveItem(CQueueItem* pItem, bool destroy, bool updateItem
 	CQueueItem* topLevelItem = pItem->GetTopLevelItem();
 
 	int count = topLevelItem->GetChildrenCount(true);
-	topLevelItem->RemoveChild(pItem, destroy);
+	topLevelItem->RemoveChild(pItem, destroy, forward);
 
 	bool didRemoveParent;
 
@@ -1451,6 +1473,35 @@ void CQueueViewBase::OnKeyDown(wxKeyEvent& event)
 		event.Skip();
 }
 
+void CQueueViewBase::WriteToFile(pugi::xml_node element) const
+{
+	auto queue = element.child("Queue");
+	if (!queue) {
+		queue = element.append_child("Queue");
+	}
+
+	for (std::vector<CServerItem*>::const_iterator iter = m_serverList.begin(); iter != m_serverList.end(); ++iter)
+		(*iter)->SaveItem(queue);
+}
+
+void CQueueViewBase::OnExport(wxCommandEvent&)
+{
+	wxFileDialog dlg(m_parent, _("Select file for exported queue"), wxString(),
+		_T("FileZilla.xml"), _T("XML files (*.xml)|*.xml"),
+		wxFD_SAVE | wxFD_OVERWRITE_PROMPT);
+
+	if (dlg.ShowModal() != wxID_OK)
+		return;
+
+	CXmlFile xml(dlg.GetPath());
+
+	auto exportRoot = xml.CreateEmpty();
+
+	WriteToFile(exportRoot);
+
+	xml.Save(true);
+}
+
 // ------
 // CQueue
 // ------
