@@ -78,7 +78,7 @@ struct sftp_packet *sftp_wait_for_reply(struct sftp_request *req)
  * links as FXP_REALPATH would resolve the link if called with the
  * full path.
  */
-char *canonify(char *name, int parent_only)
+char *canonify(const char *name, int parent_only)
 {
     char *fullname, *canonname;
     struct sftp_packet *pktin;
@@ -88,7 +88,7 @@ char *canonify(char *name, int parent_only)
     if (name[0] == '/') {
 	fullname = dupstr(name);
     } else {
-	char *slash;
+	const char *slash;
 	if (pwd[strlen(pwd) - 1] == '/')
 	    slash = "";
 	else
@@ -224,30 +224,6 @@ char *canonify(char *name, int parent_only)
     }
 }
 
-/*
- * Return a pointer to the portion of str that comes after the last
- * slash (or backslash or colon, if `local' is TRUE).
- */
-static char *stripslashes(char *str, int local)
-{
-    char *p;
-
-    if (local) {
-        p = strchr(str, ':');
-        if (p) str = p+1;
-    }
-
-    p = strrchr(str, '/');
-    if (p) str = p+1;
-
-    if (local) {
-	p = strrchr(str, '\\');
-	if (p) str = p+1;
-    }
-
-    return str;
-}
-
 /*
  * qsort comparison routine for fxp_name structures. Sorts by real
  * file name.
@@ -481,7 +457,7 @@ int sftp_get_file(char *fname, char *outfname, int recurse, int restart)
         pktin = sftp_wait_for_reply(req);
 	fxp_close_recv(pktin, req);
 
-	return 0;
+	return 2;
     }
 
     if (restart) {
@@ -713,7 +689,7 @@ int sftp_put_file(char *fname, char *outfname, int recurse, int restart)
     file = open_existing_file(fname, NULL, NULL, NULL, &permissions);
     if (!file) {
 	fzprintf(sftpError, "local: unable to open %s", fname);
-	return 0;
+	return 2;
     }
     attrs.flags = 0;
     PUT_PERMISSIONS(attrs, permissions);
@@ -1152,7 +1128,8 @@ int sftp_cmd_ls(struct sftp_command *cmd)
     struct fxp_names *names;
     struct fxp_name **ournames;
     int nnames, namesize;
-    char *dir, *cdir, *unwcdir, *wildcard;
+    const char *dir;
+    char *cdir, *unwcdir, *wildcard;
     struct sftp_packet *pktin;
     struct sftp_request *req;
     int i;
@@ -2226,7 +2203,7 @@ static int sftp_cmd_pling(struct sftp_command *cmd)
 // static int sftp_cmd_help(struct sftp_command *cmd);
 
 static struct sftp_cmd_lookup {
-    char *name;
+    const char *name;
     /*
      * For help purposes, there are two kinds of command:
      * 
@@ -2240,8 +2217,8 @@ static struct sftp_cmd_lookup {
      *    contains the help that should double up for this command.
      */
     int listed;			       /* do we list this in primary help? */
-    char *shorthelp;
-    char *longhelp;
+    const char *shorthelp;
+    const char *longhelp;
     int (*obey) (struct sftp_command *);
 } sftp_lookup[] = {
     /*
@@ -2492,7 +2469,7 @@ END FZ UNUSED */
     }
 };
 
-const struct sftp_cmd_lookup *lookup_command(char *name)
+const struct sftp_cmd_lookup *lookup_command(const char *name)
 {
     int i, j, k, cmp;
 
@@ -2742,7 +2719,7 @@ void do_sftp_cleanup()
     }
 }
 
-void do_sftp(int mode, int modeflags, char *batchfile)
+int do_sftp(int mode, int modeflags, char *batchfile)
 {
     FILE *fp;
     int ret;
@@ -2777,8 +2754,9 @@ void do_sftp(int mode, int modeflags, char *batchfile)
         fp = fopen(batchfile, "r");
         if (!fp) {
 	    printf("Fatal: unable to open %s\n", batchfile);
-	    return;
+	    return 1;
         }
+	ret = 0;
         while (1) {
 	    struct sftp_command *cmd;
 	    cmd = sftp_getcmd(fp, mode, modeflags);
@@ -2793,8 +2771,13 @@ void do_sftp(int mode, int modeflags, char *batchfile)
 	    }
         }
 	fclose(fp);
-
+	/*
+	 * In batch mode, and if exit on command failure is enabled,
+	 * any command failure causes the whole of PSFTP to fail.
+	 */
+	if (ret == 0 && !(modeflags & 2)) return 2;
     }
+    return 0;
 }
 
 /* ----------------------------------------------------------------------
@@ -2806,7 +2789,7 @@ static int verbose = 0;
 /*
  *  Print an error message and perform a fatal exit.
  */
-void fatalbox(char *fmt, ...)
+void fatalbox(const char *fmt, ...)
 {
     va_list ap;
     char* str;
@@ -2819,7 +2802,7 @@ void fatalbox(char *fmt, ...)
 
     cleanup_exit(1);
 }
-void modalfatalbox(char *fmt, ...)
+void modalfatalbox(const char *fmt, ...)
 {
     va_list ap;
     char* str;
@@ -2832,7 +2815,7 @@ void modalfatalbox(char *fmt, ...)
 
     cleanup_exit(1);
 }
-void nonfatal(char *fmt, ...)
+void nonfatal(const char *fmt, ...)
 {
     va_list ap;
     char* str;
@@ -2843,7 +2826,7 @@ void nonfatal(char *fmt, ...)
     fzprintf(sftpError, str);
     sfree(str);
 }
-void connection_fatal(void *frontend, char *fmt, ...)
+void connection_fatal(void *frontend, const char *fmt, ...)
 {
     va_list ap;
     char* str;
@@ -3083,6 +3066,7 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
         conf_set_int(conf, CONF_port, 22);
     }
 
+#if 0
     /*
      * If saved session / Default Settings says SSH-1 (`1 only' or `1'),
      * then change it to SSH-2, on the grounds that that's more likely to
@@ -3091,6 +3075,10 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
      */
     if ((conf_get_int(conf, CONF_sshprot) & ~1) != 2)   /* is it 2 or 3? */
 	conf_set_int(conf, CONF_sshprot, 2);
+#else
+    /* FZ: Require SSH2 */
+    conf_set_int(conf, CONF_sshprot, 3);
+#endif
 
     /*
      * Enact command-line overrides.
@@ -3225,7 +3213,7 @@ static int psftp_connect(char *userhost, char *user, int portnumber)
     return 0;
 }
 
-void cmdline_error(char *p, ...)
+void cmdline_error(const char *p, ...)
 {
     char *str;
     va_list ap;
@@ -3333,7 +3321,7 @@ const int share_can_be_upstream = FALSE;
  */
 int psftp_main(int argc, char *argv[])
 {
-    int i;
+    int i, ret;
     int portnumber = 0;
     char *userhost, *user;
     int mode = 0;
@@ -3366,6 +3354,34 @@ int psftp_main(int argc, char *argv[])
     // FZ: Set proxy to none
     conf_set_int(conf, CONF_proxy_type, PROXY_NONE);
 
+    // FZ: Re-order ciphers so that old and insecure algorithms are always below the warning level
+    {
+	// Find position of warning level
+	int warn = -1;
+	for (i = 0; i < CIPHER_MAX && warn == -1; ++i) {
+	    int cipher = conf_get_int_int(conf, CONF_ssh_cipherlist, i);
+	    if (cipher == CIPHER_WARN) {
+		warn = i;
+	    }
+	}
+
+	if (warn != -1 && warn < CIPHER_MAX) {
+	    for (i = warn - 1; i >= 0; --i) {
+		int const cipher = conf_get_int_int(conf, CONF_ssh_cipherlist, i);
+		if (cipher == CIPHER_ARCFOUR || cipher == CIPHER_DES) {
+		    int j;
+		    // Bubble it down
+		    for (j = i; j < warn; ++j) {
+			int swap = conf_get_int_int(conf, CONF_ssh_cipherlist, j + 1);
+			conf_set_int_int(conf, CONF_ssh_cipherlist, j, swap);
+		    }
+		    conf_set_int_int(conf, CONF_ssh_cipherlist, warn, cipher);
+		    --warn;
+		}
+	    }
+	}
+    }
+
     for (i = 1; i < argc; i++) {
 	int ret;
 	if (argv[i][0] != '-') {
@@ -3444,7 +3460,7 @@ int psftp_main(int argc, char *argv[])
 //	       " to connect\n");
     }
 
-    do_sftp(mode, modeflags, batchfile);
+    ret = do_sftp(mode, modeflags, batchfile);
 
     if (back != NULL && back->connected(backhandle)) {
 	char ch;
@@ -3458,23 +3474,5 @@ int psftp_main(int argc, char *argv[])
     console_provide_logctx(NULL);
     sk_cleanup();
 
-/*    {
-	Keyfile_list *list = cfg.keyfile_list;
-	while (list)
-	{
-	    Keyfile_list *next = list->next;
-	    sfree(list);
-	    list = next;
-	}
-    }*/
-
-    return 0;
+    return ret;
 }
-
-/*void notify_remote_exit(void *frontend)
-{
-    int exitcode = back->exitcode(backhandle);
-
-    if (exitcode > 0 && exitcode != INT_MAX)
-	connection_fatal(frontend, "Connection closed by server with exitcode %d", exitcode);
-}*/
